
# Script for plotting
import matplotlib.pyplot as plt
from interpolate_unit_convert import interpolation as int
from read_model import file1_read
from optimizer import optimize as opt
import random

class plotting:
    def __init__(self):
        self.dict = file1_read[0]

    def get_x(self, FreeVar_num):
        file = int()
        x = file.get_points()[FreeVar_num]
        return x

    def define_y2(self, x_axis, TNP, GOR, WC, GLR, LR):
        file = int()
        interpol = file.do_interpolation(0)
        string = x_axis.lower()
        if string == 'lr':
            x = self.get_x(4)
            y = []
            for i in range(len(x)):  # np.size
                input_values = (TNP, GOR, WC, GLR, x[i])
                res = interpol(input_values)
                y.append(res)
        elif string == 'glr':
            x = self.get_x(3)
            y = []
            for i in range(len(x)):
                input_values = (TNP, GOR, WC, x[i], LR)
                res = interpol(input_values)
                y.append(res)
        else: y = 'The input for x-axis is not valid. Insert LR or GLR (not case sensitive)'
        return y

    def plot(self, X, Y, title, xlabel, ylabel):
        plt.plot(X, Y, '--bo')
        plt.title(title)
        plt.xlabel(xlabel)
        plt.ylabel(ylabel)
        plt.show()
        return plt

    def plot_GLR_LRvsWHP(self, xaxis, TNP, GOR, WC, GLR, LR):
        xax = xaxis.lower()
        if xax == 'lr':
            x = self.get_x(4)
            y = self.define_y2(xaxis, TNP, GOR, WC, GLR, LR)  # The input x-axis decides what values the y should be plotted against, LR or GLR
            plt = self.plot(x, y, 'WHP vs liquid rate', 'Liquid rate [Sm^3/day ?]', 'BHP [bar]')
        elif xax == 'glr':
            x = self.get_x(3)
            y = self.define_y2(xaxis, TNP, GOR, WC, GLR, LR)  # The input x-axis decides what values the y should be plotted against, LR or GLR
            plt = self.plot(x, y, 'WHP vs Gas lift injection rate', 'Gas lift inj. rate rate [Sm^3/day ?]', 'BHP [bar]')
        else: plt = 'The input for x-axis is not valid. Insert LR or GLR (not case sensitive)'
        return plt

    def multiple_plots(self, list, type):  # type, BHP, GOR, WC, GLR, LR,
        file1 = int()
        interpol = file1.do_interpolation(0)
        if type == 'glr':
            x = self.get_x(3)
            for i in range(len(list)):
                y_list = []
                for j in range(len(x)):
                    input_values = (120, 91, 50, x[j], list[i])
                    res = interpol(input_values)
                    y_list.append(res)
                r = random.random()
                g = random.random()
                b = random.random()
                color = (r, g, b)
                plt.plot(x, y_list, '--o', c = color, label=f'LR = {list[i]}')
                # plt.legend(loc='upper left')
            # Define straight line
            line = []
            for i in range(len(x)):
                line.append(20)
            plt.plot(x, line, '--', c = (1, 0, 0), label = 'Required pressure')
            plt.legend(loc='upper left')
            plt.xlabel('Gas lift rate [Sm^3/day]')
            plt.ylabel('WHP [bar]')
            # plt.title('WHP with varying GLR shown for different values of Liquid rate')
        elif type == 'lr':
            x = self.get_x(4)
            for i in range(len(list)):
                y_list = []
                for j in range(len(x)):
                    input_values = (120, 91, 50, list[i], x[j])
                    res = interpol(input_values)
                    y_list.append(res)
                r = random.random()
                g = random.random()
                b = random.random()
                color = (r, g, b)
                plt.plot(x, y_list, '--o', c = color, label=f'GLR = {list[i]}')
                # plt.legend(loc='upper left')
            # Define straight line
            line = []
            for i in range(len(x)):
                line.append(20)
            plt.plot(x, line, '--', c = (1, 0, 0), label = 'Required pressure')
            plt.legend(loc='upper right')
            plt.xlabel('Liquid rate [Sm^3/day]')
            plt.ylabel('WHP [bar]')
            # plt.title('WHP with varying GLR shown for different values of Liquid rate')
        return None

    def plot_GLRvsLR(self, BHP, GOR, WC, pressure_cons_val, remove):
        LR = self.get_x(4)
        LR_new = LR[:-remove]  # adding this to remove last items
        oil_rate_list = []
        for i in range(len(LR_new)):  # Make new list with oil rate instead of Liquid rate
            oil_rate = LR_new[i ] *(1 - W C /100)
            oil_rate_list.append(oil_rate)
        file = opt()
        x0 = (0)
        res_list = []
        for i in range(len(LR_new)):
            res = file.min_GLR(x0, LR_new[i], BHP, GOR, WC, pressure_cons_val)
            res_list.append(res.x)
        r = random.random()
        g = random.random()
        b = random.random()
        color = (r, g, b)
        plt.plot(res_list, oil_rate_list, '--o', label=f'WC = {WC}')
        plt.legend(loc='upper left')
        plt.xlabel('Gas lift rate [Sm^3/day]')
        plt.ylabel('Oil rate [Sm^3/day]')
        return plt

    def plot_runningtime(self, x, y, x1, y1, xlab, ylab):
        plt.plot(range(len(x)), y, '--o')
        plt.plot(range(len(x1)), y1, '--o')
        plt.xlabel(xlab)
        plt.ylabel(ylab)
        plt.xticks(range(len(x)), x)
        plt.legend(['Data set', 'Quadratic Running Time'])
        return plt
## Test of plotting 1 - GLR/LR vs BHP for one constant
file = plotting()
# res = file.plot_GLR_LRvsWHP('glr', 120, 91, 50, 28000, 2000) #Order: LR/GLR, TNP, GOR, WC, GLR, LR: If Liquid rate is chosen, ...
# LR-value can be chosen arbitraraly and opposite: if Gas lift rate is chosen,  GLR-value can be chosen arbitraraly. The other values has to be chosen within bounds

## Test of plotting 2 - GLR/LR vs BHP for multiple constants
# LR = file.get_x(4)
# print(LR)
# New_LR = LR[0::2]
# New_LR1 = np.delete(New_LR, [1, 2, 3]) #Remove some items

# GLR = file.get_x(3)
# res = file.multiple_plots(GLR, 'lr') #if last is lr, first input has to be GLR, and opposite: if input 2 = 'glr, first input should be New_LR1 or LR
# plt.savefig("WHP_vs_LR_forDifferentGLR1.pdf", bbox_inches='tight') #bbox_inches='tight'
# plt.show()
# for i in range(4):
#     print(file.get_x(i))

## Test of plotting 3 - LR vs GLR for 20 bar
# plot = file.plot_GLRvsLR(150, 91, 70, 20)
# #plt.savefig("LR_vs_GLR_pdf_BHP150_GOR91_WC70_1.pdf", bbox_inches='tight') #bbox_inches='tight'
# plt.show()

## Test of plotting 4 - LR vs GLR for 20 bar for multiple values of WC:
WC = [40, 45, 50, 55, 60]
for i in range(len(WC)):
    if WC[i] == 40:
        rem = 1
    elif WC[i] == 45:
        rem = 1
    elif WC[i] == 50:
        rem = 2
    elif WC[i] == 55:
        rem = 2
    elif WC[i] == 60:
        rem = 3
    file.plot_GLRvsLR(120, 91, WC[i], 20, rem)
# plt.savefig("LR_vs_GLR_pdf_BHP120_GOR91_different_WC_1.pdf", bbox_inches='tight') #bbox_inches='tight'
plt.show()

## Test of plotting 5 - Running time

# file.plot_runningtime([1, 2, 3, 4, 5, 6, 7, 8], [0.5, 1.5, 5, 10, 13, 26, 47, 71], [1, 2, 3, 4, 5, 6, 7, 8], [1, 4, 9, 16, 25, 36, 49, 64], 'Number of wells', 'Running time (seconds)')
# plt.savefig("runningtime_for8wells_quad.pdf", bbox_inches='tight') #bbox_inches='tight'
# plt.show()